//////////////////////////////////////////////////////////////////////
// Water Drops Demo by KeithE
// Updated January 30, 2007 to use libnds DS Motion Card functions
// based on Simple ARM9 demo (touch screen controls the colors)
// -- joat
//////////////////////////////////////////////////////////////////////

#include <nds.h>
#include <stdio.h>
#include <nds/memory.h>
//#include <nds/arm9/rand.h>
#include <nds/arm9/video.h>
#include <nds/arm9/trig_lut.h>
#include <stdlib.h>
#include <math.h>
#include <nds/arm9/ndsmotion.h>

#define VRAM_ARM9 (1)
#define VRAM_CORE_A 0x06000000
#define VRAM_CORE_B 0x06040000
#define DISP_SR (*(vuint16*)0x04000004)

//////////////////////////////////////////////////////////////////////
void CirclePoints (int x, int y, int value, uint16 *vram);
void MidpointCircle ( int xc, int yc, int radius, int value, uint16 *vram );
void WritePixel(int x, int y , int value, uint16 *vram);
void pltpoint(int x, int y, int c, uint16 *vram);
void circle(int x, int y, int r, int c, uint16 *vram);

volatile unsigned int counter = 0;
void InterruptHandler(void) {

  if (REG_IF & IRQ_VBLANK) {
    counter++;

//    PALETTE[0] = RGB15(IPC->touchX>>7, IPC->touchY>>7, (counter>>4)&0x1F);
//    PALETTE[512] = RGB15(15, IPC->touchZ1>>7, IPC->touchZ2>>7);

    REG_IF = IRQ_VBLANK;
  }
}


//////////////////////////////////////////////////////////////////////

#define KEY_TOUCH (((~IPC->buttons) << 6) & (1<<12))

struct drop
{
    float x;
    float y;
    int c;
    int r;
    int speed;
};

struct drop drops[128];

int main(int argc, char ** argv) {
  // Red main screen, blue sub-screen
  PALETTE[0] = RGB15(0, 0, 31);
//  PALETTE[512] = RGB15(5, 5, 15);

  // Turn on the screens and 2D cores and switch to mode 0
  REG_POWERCNT = POWER_ALL_2D;
  DISPLAY_CR = MODE_FB0;
  SUB_DISPLAY_CR = MODE_0_2D | DISPLAY_BG0_ACTIVE;
    VRAM_A_CR = VRAM_ENABLE | VRAM_ARM9 | VRAM_CORE_A;
    VRAM_B_CR = VRAM_ENABLE | VRAM_ARM9 | VRAM_CORE_A;

    VRAM_C_CR = VRAM_ENABLE | VRAM_ARM9 | VRAM_CORE_B;
    VRAM_D_CR = VRAM_ENABLE | VRAM_ARM9 | VRAM_CORE_B;

    SUB_BG0_CR = BG_COLOR_256 | (1 << SCREEN_SHIFT);
    SUB_BG1_CR = BG_COLOR_256 | (2 << SCREEN_SHIFT);


    vramSetMainBanks(VRAM_A_LCD,VRAM_B_LCD,VRAM_C_SUB_BG,VRAM_D_SUB_SPRITE);
	
	SUB_BG0_CR = BG_MAP_BASE(31);
	
	BG_PALETTE_SUB[255] = RGB15(0,0,31);	//font will be rendered in blue
	BG_PALETTE_SUB[0] = 0;	//black background

	
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);

  // Enable the V-blank interrupt
  REG_IME = 0;
  IRQ_HANDLER = &InterruptHandler;
  REG_IE = IRQ_VBLANK;
  REG_IF = ~0;
  DISP_SR = DISP_VBLANK_IRQ;
  REG_IME = 1;
    
	iprintf("\x1b[1;0HInsert DS Motion Card - Press B");
	
	scanKeys();
	u32 keys = keysDown();
	while(!(keys & KEY_B))
	{
		scanKeys();
		keys = keysDown();
	}
	
	iprintf("\x1b[1;0H                               ");
  
	// enable the DS Motion Card
	motion_enable();

	// check whether a DS Motion Card is plugged in
	int ndsmotion = motion_is_inserted();
	
	// if there is no DS Motion Card, the DSaSketch will not work
	if(ndsmotion == 0) {
		BG_PALETTE_SUB[255] = RGB15(31,0,0); // RED font
		BG_PALETTE_SUB[0] = RGB15(0,0,0);	//black background
		// set the entire main to black
		iprintf("\x1b[1;0H  DS Motion Card not detected!");
		while(1)
		{
		}
	}
	else { // if there is a DS Motion Card, then continue
	
	iprintf("\n  Draw drops with the stylus\n Tilt to roll the drops around");
  
  int tx=0;
  int ty=0;
  
  int touched=0;

	int i = 0;
  for(i=0; i<128; i++)
  {
      drops[i].r = 0;
  }

  for(i=0; i<256*192; i++)
  {
      VRAM_C[i] = RGB15(5,5,15);
  }


  // Touchscreen calibration
  int divx = 14;
  int divy = 18;
  int subx = 00;
  int suby = 12;
  uint16* vram = VRAM_A;

  int tx2=0;
  int ty2=0;
  
  	int Xoutput = 2048;
	int Youtput = 2048;
	float Xmove[128] = {0};
	float Ymove[128] = {0};
	float bounce = 0.85;
	
	touchPosition touch;
  

		
  while (1) 
  {
  
	
		
//      swiWaitForVBlank();
//      counter++;
      if(counter > 0)
      {
	    Xoutput = motion_read_x();
		Youtput = motion_read_y();
		
          counter = 0;
          
          if(KEY_TOUCH)
          {
              if(0 == 0)
              {
                  tx2 = tx;
                  ty2 = ty;
                  touched = 1;
				  touch=touchReadXY();
                  tx = touch.px; // IPC->touchX / divx - subx;
                  ty = touch.py; // IPC->touchY / divy - suby;

                  if(tx2 != tx || ty2 != ty)
                  {
                      int n=-1;
                      
                      for(i=0; i<128; i++)
                      {
                          if(drops[i].r <= 0)
                          {
                              n=i;
                              break;
                          }
                      }
                      if(n != -1)
                      {
                          drops[n].x = tx;
                          drops[n].y = ty;
                          drops[n].r = 10;
                          drops[n].speed = 8 + rand()%5;
                          drops[n].c = rand()%8;
						  Xmove[n] = 0;
						  Ymove[n] = 0;
                      }
                  }
                      
              }
          }
          else
              touched = 0;
          
          dmaCopyWords(3, (uint32 *)VRAM_C, (uint32 *)vram, 256*192*2);
/*          for(int i=0; i<256*192; i++)
          {
              vram[i] = 0;
          }*/
		  int n = 0;
          for(n=0; n<128; n++)
          {
              if(drops[n].r > 0)
              {
                  //vram[n*2] = RGB15(0,0,31); // drawing line on top
                  //vram[n*2 + 1] = RGB15(0,0,31); // drawing line on top
//                  vram[512 + n] = RGB15(0,0,31);
                  drops[n].r+=drops[n].speed;
				  Xmove[n] += ((float)Xoutput-2048)/2500;
				  Ymove[n] += ((float)Youtput-2048)/2500;
				  drops[n].x -= Xmove[n];
				  drops[n].y += Ymove[n];
				  
				// check if reached edge of screen
				if(drops[n].x-drops[n].r/16 <=0)
				{
					drops[n].x = drops[n].r/16;
					Xmove[n] = -Xmove[n]*bounce;
				}
				
				if(drops[n].x+drops[n].r/16 >=255)
				{
					drops[n].x = 255-drops[n].r/16;
					Xmove[n] = -Xmove[n]*bounce;
				}
				if(drops[n].y-drops[n].r/16 <=0)
				{
					drops[n].y = drops[n].r/16;
					Ymove[n] = -Ymove[n]*bounce;
				}
				
				if(drops[n].y+drops[n].r/16 >=191)
				{
					drops[n].y = 191-drops[n].r/16;
					Ymove[n] = -Ymove[n]*bounce;
				}				
				  
			
                  int x = (int)(drops[n].x+.5);
                  int y = (int)(drops[n].y+.5);
                  int r = drops[n].r;
                  int c = drops[n].c;
  
                  circle(x,y,r/16,         RGB15(c+15-(r/64), c+15-r/64, 31-r/64), vram);        
/*
                  for(int i = 0; i<r; i++)
                  {
                      vram[(y-r/2)*256 + x + i - r/2] = RGB15(31-r/4, 31-r/4, 31-r/4);
                      vram[(y+r/2)*256 + x + i - r/2] = RGB15(31-r/4, 31-r/4, 31-r/4);

                      vram[(y-r/2 + i)*256 + x - r/2] = RGB15(31-r/4, 31-r/4, 31-r/4);
                      vram[(y-r/2 + i)*256 + x + r/2] = RGB15(31-r/4, 31-r/4, 31-r/4);
                  }*/


                  if(drops[n].r >= (63*16))
                  {
                      drops[n].r = -1;
                  }
              }

          }
         
          if(vram == VRAM_A)
          {
              videoSetMode(MODE_FB0);
              vram = VRAM_B;
          }
          else
          {
              videoSetMode(MODE_FB1);
              vram = VRAM_A;
          }
          
      }
      
      
      
      
      
  }
  } // end else
  return 0;
}

void circle(int x, int y, int r, int c, uint16 *vram)
{	int a = 0;
    for(a=0; a<512; a+=4)
    {
            pltpoint(x+((COS[a]*r)>>12),y+((SIN[a]*r)>>12),c,vram);
    }
}

void pltpoint(int x, int y, int c, uint16 *vram)
{
    int n = y*256+x;
    if(n > 256*192 || n<0)
        return;
//    if(y >= 192 || y < 0 || x >= 256 || x < 0)
  //      return;
    vram[y*256 + x] = c;
}


//////////////////////////////////////////////////////////////////////
